<?php

namespace App\Controller;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

class Integrations
{
    private $integrationsFactory;

    public function __construct(\App\Factory\Integrations $integrationsFactory)
    {
        $this->integrationsFactory = $integrationsFactory;
    }

    public function getCdIntegracaoByIdIntegracao(Request $request, Response $response) {
        try {
            $params = $request->getQueryParams();
            $result = $this->integrationsFactory->getCdIntegracaoByIdIntegracao($params['idIntegracao']);

            $response->getBody()->write(
                json_encode([
                    "error" => false,
                    "message" => "",
                    "cdIntegracao" => $result,
                ])
            );
            return $response->withHeader("Content-Type", "application/json");
         } catch(\Exception $e) {
            $response->getBody()->write(
                json_encode([
                    "error" => true,
                    "message" => $e->getMessage(),
                    "data" => ""
                ])
            );

         }
        return $response->withHeader("Content-Type", "application/json");
    }

    public function getOccurrences(Request $request, Response $response) {
        $params = $request->getQueryParams();
        $result = $this->integrationsFactory->getOccurrences($params);

        if (empty($result)) {
            $response->getBody()->write(
                json_encode([
                    "error" => true,
                    "message" => "Nenhum item foi encontrado.",
                    "data" => [],
                    "pagination" => [
                        "limit" => $params["limit"] ?? 10,
                        "page" => $params["page"] ?? 1,
                    ],
                ])
            );
        } else {
        $response->getBody()->write(
            json_encode([
                "error" => false,
                "message" => "",
                "data" => $result[0],
                "pagination" => [
                    "limit" => $params["limit"] ?? 10,
                    "page" => $params["page"] ?? 1,
                    "total" => $result[1][0]["TOTAL"],
                ],
            ])
        );
    }
        return $response->withHeader("Content-Type", "application/json");
    }

    public function getProducts(Request $request, Response $response) {
     $params = $request->getQueryParams();
     $result = $this->integrationsFactory->getProducts($params['search'], $params['page']??1, $params['limit'] ?? 15);

     if (empty($result)) {
         $response->getBody()->write(
             json_encode([
                 "error" => true,
                 "message" => "Nenhum item foi encontrado.",
                 "data" => [],
                 "pagination" => [
                     "limit" => $params["limit"] ?? 10,
                     "page" => $params["page"] ?? 1,
                 ],
             ])
         );
     } else {

     $response->getBody()->write(
         json_encode([
             "error" => false,
             "message" => "",
             "data" => $result[0],
             "pagination" => [
                 "limit" => $params["limit"] ?? 10,
                 "page" => $params["page"] ?? 1,
                 "total" => $result[1][0]["TOTAL"],
             ],
         ])
     );
 }
     return $response->withHeader("Content-Type", "application/json");
 }

    public function getAlterationLogs(Request $request, Response $response) {
     try {
         $formattedResult = [];
         $params = $request->getQueryParams();
         $result = $this->integrationsFactory->getAlterationLogs($params);
         foreach($result[0] as $row) {
            $formattedResult = array_merge($formattedResult, $this->formattedLogResult($row));
         }
         if (empty($result)) {
             $response->getBody()->write(
                 json_encode([
                     "error" => true,
                     "message" => "Nenhum item foi encontrado.",
                     "data" => [],
                     "pagination" => [
                         "limit" => $params["limit"] ?? 10,
                         "page" => $params["page"] ?? 1,
                     ],
                 ])
             );
         } else {
         $response->getBody()->write(
             json_encode([
                 "error" => false,
                 "message" => "",
                 "data" => $formattedResult,
                 "pagination" => [
                    "limit" => $params["limit"] ?? 10,
                    "page" => $params["page"] ?? 1,
                    "total" => $result[1]??null,
                 ]
             ])
         );
     }

         return $response->withHeader("Content-Type", "application/json");
     } catch (\Exception $e) {
         $response->getBody()->write(
             json_encode([
                 "error" => true,
                 "message" => $e->getMessage(),
                 "data" => [],
             ])
         );

         return $response->withHeader("Content-Type", "application/json");
     }

     return $result;

 }

 public function checkSkuExsists(Request $request, Response $response) {
     try {
        $params = $request->getQueryParams();
        $cdProduto = $this->integrationsFactory->checkSkuExists($params['sku']);

        $response->getBody()->write(
            json_encode([
                "error" => false,
                "message" => "",
                "cdProduto" => $cdProduto,
            ])
        );
        return $response->withHeader("Content-Type", "application/json");
     } catch(\Exception $e) {
        $response->getBody()->write(
            json_encode([
                "error" => true,
                "message" => $e->getMessage(),
                "data" => ""
            ])
        );

     }
    return $response->withHeader("Content-Type", "application/json");
 }

 public function checkOccurrenceExists(Request $request, Response $response) {
    try {
        $params = $request->getQueryParams();
        $cdProduto = $this->integrationsFactory->checkOccurrenceExists($params);

        $response->getBody()->write(
            json_encode([
                "error" => false,
                "message" => "",
                "cdProduto" => $cdProduto,
            ])
        );
        return $response->withHeader("Content-Type", "application/json");
     } catch(\Exception $e) {
        $response->getBody()->write(
            json_encode([
                "error" => true,
                "message" => $e->getMessage(),
                "data" => ""
            ])
        );

     }
    return $response->withHeader("Content-Type", "application/json");
 }

 private function formattedLogResult($row) {

    if ($row['ANTERIOR_VRPREC'] != $row['ATUAL_VRPREC'] &&
    $row['ANTERIOR_CDEXT'] != $row['ATUAL_CDEXT']) {
    $result = [];

    $result[] = [
        'CDLOG'      => $row['CDLOG'],
        'DTINCLUSAO' => $row['DTINCLUSAO'],
        'AUTOR'      => $row['AUTOR'],
        'DSALTERA'   => $row['NMPRODCARD'],
        'ANTERIOR'   => null,
        'ATUAL'      => null,
    ];
    $result[] = [
        'CDLOG'      => $row['CDLOG'].'(1)',
        'DTINCLUSAO' => null,
        'PARENT_ID'  => $row['CDLOG'],
        'AUTOR'      => null,
        'DSALTERA'   => 'Preço Alterado.',
        'ANTERIOR'   => $this->formatPrice($row['ANTERIOR_VRPREC']),
        'ATUAL'      => $this->formatPrice($row['ATUAL_VRPREC']),
    ];
    $result[] = [
        'CDLOG'      => $row['CDLOG'].'(2)',
        'DTINCLUSAO' => null,
        'PARENT_ID'  => $row['CDLOG'],
        'AUTOR'      => null,
        'DSALTERA'   => 'Código Externo Alterado.',
        'ANTERIOR'   => $row['ANTERIOR_CDEXT'],
        'ATUAL'      => $row['ATUAL_CDEXT'],
    ];
} else {
    $previousValue = '';
    $newValue = '';

    if(!empty($row['ANTERIOR_VRPREC']) && empty(!$row['ATUAL_VRPREC'])){
        $previousValue = $this->formatPrice($row['ANTERIOR_VRPREC']);
        $newValue =   $this->formatPrice($row['ATUAL_VRPREC']);
    }

    if(!empty($row['ANTERIOR_CDEXT']) && empty(!$row['ATUAL_CDEXT'])){
        $previousValue = $row['ANTERIOR_CDEXT'];
        $newValue =   $row['ATUAL_CDEXT'];
    }

    $result[] = [
        'CDLOG'      => $row['CDLOG'],
        'DTINCLUSAO' => $row['DTINCLUSAO'],
        'AUTOR'      => $row['AUTOR'],
        'DSALTERA'   => $row['DSALTERA'],
        'ANTERIOR'   => $previousValue,
        'ATUAL'      => $newValue,
    ];
    }
    return $result;
}

    function formatPrice($price) {

        $formattedPrice = 'R$'.number_format($price, 2, ',', '.');

        return $formattedPrice;
    }

    public function getSubsidiaryByIntegrationCode(Request $request, Response $response)
    {
        try {
            $params = $request->getQueryParams();
            $result = $this->integrationsFactory->getSubsidiaryByIntegrationCode($params['cdIntegracao']);

            $response->getBody()->write(
                json_encode([
                    "error" => false,
                    "message" => "",
                    "data" => $result,
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        } catch (\Exception $e) {
            $response->getBody()->write(
                json_encode([
                    "error" => true,
                    "message" => $e->getMessage(),
                    "data" => [],
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        }
    }

    public function getSubsidiaryIdToken(Request $request, Response $response)
    {
        try {
            $params = $request->getQueryParams();
            $result = $this->integrationsFactory->getSubsidiaryIdToken($params);

            $response->getBody()->write(
                json_encode([
                    "error" => false,
                    "message" => "",
                    "data" => $result,
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        } catch (\Exception $e) {
            $response->getBody()->write(
                json_encode([
                    "error" => true,
                    "message" => $e->getMessage(),
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        }
    }

}
