<?php
namespace App\Controller;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use App\Report\ReportBuilder;

class Report {
  private $reportService;

  public function __construct(\App\Service\Report $reportService) {
    $this->reportService = $reportService;
  }

  public function generateReport(Request $request, Response $response) {
    try {
      $params = $request->getParsedBody();
      $report = $this->reportService->generateReport($params);

      $response->getBody()->write($report);
      if ($params['format'] === 'PDF') {
        return $response->withHeader('Content-Type', 'application/pdf');
      }
      return $response->withHeader('Content-Type', 'application/json');
    } catch (\Exception $e) {
      $response->getBody()->write(json_encode([
        'error'   => true,
        'message' => $e->getMessage(),
        'data'    => []
      ]));
      
      return $response->withHeader('Content-Type', 'application/json');
    }
  }

  public function getSector(Request $request, Response $response) {
    try {
      $params = $request->getQueryParams();
      $sectors = $this->reportService->getSector($params);
      $fixedData = [
          'CDSETOR' => '0',
          'NMSETOR' => 'Outros',
      ];
      if (is_array($sectors) && !empty($sectors)) {
        array_unshift($sectors, $fixedData);
      }
      $response->getBody()->write(json_encode([
        'error' => false,
        'message' => '',
        'data' => $sectors ?? [$fixedData],
        'pagination' => [
          'limit' => $params['limit'] ?? 50,
          'page' => $params['page'] ?? 1
      ]
    ]));

    return $response->withHeader('Content-Type', 'application/json');
    } catch (\Exception $e) {
      $response->getBody()->write(json_encode([
        'error'   => true,
        'message' => $e->getMessage(),
        'data'    => []
    ]));

      return $response->withHeader('Content-Type', 'application/json');
    }
  }

  public function getUnit(Request $request, Response $response) {
    try {
      $params = $request->getQueryParams();
      $units = $this->reportService->getUnit($params);

      $response->getBody()->write(json_encode([
        'error' => false,
        'message' => '',
        'data' => $units ?? [],
        'pagination' => [
          'limit' => $params['limit'] ?? 50,
          'page' => $params['page'] ?? 1
      ]
    ]));

    return $response->withHeader('Content-Type', 'application/json');
    } catch (\Exception $e) {
      $response->getBody()->write(json_encode([
        'error'   => true,
        'message' => $e->getMessage(),
        'data'    => []
    ]));

      return $response->withHeader('Content-Type', 'application/json');
    }
  }
}